import { AbstractProvider } from '@pokt-foundation/pocketjs-abstract-provider';
import { TransactionResponse } from '@pokt-foundation/pocketjs-types';
import { AbstractSigner, TransactionRequest } from './abstract-signer';
import { KeyManager } from './key-manager';
/**
 * A Wallet is a minimal wallet implementation that lets you import accounts
 * with their private key or PPK (Portable Private Key), get its address, public key, private key, sign messages
 * and query its own information from the chain if a provider is attached.
 **/
export declare class Wallet implements AbstractSigner {
    readonly provider?: AbstractProvider;
    readonly _isSigner: boolean;
    readonly keyManager: KeyManager;
    constructor({ provider, keyManager, isSigner, }: {
        provider?: AbstractProvider;
        keyManager: KeyManager;
        isSigner: boolean;
    });
    /**
     * Creates a new, random Pocket account.
     * @returns {Wallet} - A new Wallet instance with the account attached.
     * */
    static createRandom(): Promise<Wallet>;
    /**
     * Instanciates a new Wallet from a valid ED25519 private key.
     * @param {string} privateKey - The private key to use to instanciate the new Key manager.
     * @returns {Wallet} - A new Key Manager instance with the account attached.
     * */
    static fromPrivateKey(privateKey: string): Promise<Wallet>;
    /**
     * Instanciates a new Wallet from a valid PPK.
     * @param {string} privateKey - The private key to use to instanciate the new Key manager.
     * @returns {Wallet} - A new Key Manager instance with the account attached.
     * */
    static fromPPK({ password, ppk, }: {
        password: string;
        ppk: string;
    }): Promise<Wallet>;
    /**
     * Exports a private key as a Portable-Private-Key, unlockable with the used password.
     * @param {string} password - The password to use in the PPK.
     * @param {string} hint - Password hint.
     * @returns {KeyManager} - A new Key Manager instance with the account attached.
     * */
    exportPPK({ password, hint, }: {
        password: string;
        hint?: string;
    }): Promise<string>;
    /**
     * Gets the account address.
     * @returns {string} - The attached account's address.
     * */
    getAddress(): Promise<string>;
    /**
     * Attaches a provider to the Wallet.
     * @returns {Wallet} - A clone instance of the Wallet with the provider attached.
     * */
    connect(provider: AbstractProvider): Wallet;
    /**
     * Fetches the wallet's account balance. A provider is required.
     * @returns {bigint} - The wallet account's balance.
     * */
    getBalance(): Promise<bigint>;
    /**
     * Fetches the wallet's account transaction count. A provider is required.
     * @returns {number} - The wallet account's transaction count.
     * */
    getTransactionCount(): Promise<number>;
    /**
     * Sends a transaction from this wallet's account. A provider is required.
     * @returns {Transaction} - The network's response to the transaction.
     * */
    sendTransaction(signedTransaction: string | Promise<string>): Promise<TransactionResponse>;
    /**
     * signs a valid hex-string payload with the imported private key.
     * @param {string} payload - The hex payload to sign.
     * @returns {string} - The signed payload as a string.
     * */
    sign(payload: string): Promise<string>;
    /**
     * Signs a transaction from this wallet's account.
     * @param {TransactionRequest} transaction - The transaction to sign, formatted as a `TransactionRequest`.
     * @returns {string} - The signed transaction.
     * */
    signTransaction(transaction: TransactionRequest): Promise<string>;
}
//# sourceMappingURL=wallet.d.ts.map